<?php

namespace App\Http\Controllers;

use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class UserController extends Controller
{
  // views

  public function usersView()
  {
    // needed info (besides id's):
    // user: name
    // userGoal: goal_active, goal_achieved
    $users = User::getWithActiveGoals();
    
    return view('admin.home', ['users' => $users]);
  }

  // CRUD

  public function usersRead()
  {
    // needed info (besides id's):
    // user: name, can_select_goals, add goals_approved
    // userGoal: goal_achieved
    // goal: title, add skill title
    // also, i would prefer the booleans to be true & false instead of 1 & 0. (for js)
    // this stuff is done in js, i don't know how to do it in laravel

    $users = User::getWithActiveGoalsAndGoalAndSkillTitles();

    return $users;
  }

  public function updateUserData(Request $request) {
    // needed info (besides id's):
    // user: can_select_goals
    // userGoal: goal_approved, goal_achieved

    $user = User::select('id', 'can_select_goals')->with(['goals' => function($query) {
      $query->select('id', 'user_id', 'goal_approved', 'goal_achieved');
    }])->find($request->userId);
    
    // turn true & false back into 1 & 0
    $dataCanSelectGoals = $request->data['canSelectGoals'] === true ? 1 : 0;
    $dataGoalsApproved = $request->data['goalsApproved'] === true ? 1 : 0;

    // update can_select_goals
    if($user->can_select_goals != $dataCanSelectGoals) {
      if($dataCanSelectGoals === 1) {
        $user->canSelectGoals();
      } else {
        $user->canNotSelectGoals();
      }
    }

    // update approved goals
    if($dataGoalsApproved === 1) {
      $userGoals = $user->goals->where('goal_approved', 0);
  
      foreach($userGoals as $userGoal) {
        $userGoal->approveGoal();
      }
    }

    foreach($request->data['goals'] as $goal) {
      // delete deleted goals
      if($goal['isDeleted']) {
        $targetGoal = $user->goals->find($goal['id']);
        if($targetGoal != null) $targetGoal->delete();
      }
      
      // update achieved goals
      if($goal['goalAchieved'] === true) {
        $targetGoal = $user->goals->where('goal_achieved', 0)->find($goal['id']);
        if($targetGoal != null) $targetGoal->achieveGoal();
      } else {
        $targetGoal = $user->goals->where('goal_achieved', 1)->find($goal['id']);
        if($targetGoal != null) $targetGoal->unachieveGoal();
      }
    }
  }
  
}
