"use strict"

import "../components/tabNavigation.js"


// declarations
const goalsList = document.querySelector('#goals-list')

const selectedGoals = document.querySelector('#selected-goals')
const selectedList = document.querySelector('#selected-list')
const selectedGoalsCount = document.querySelector('#selected-goals-count')
const noItemsMessage = document.querySelector('#no-items-message')

const modalBackground = document.querySelector('#modal-background')
const modalSelected = document.querySelector('#modal-selected-goals')
const modalSelectedList = document.querySelector('#selected-goals-list')

let selectedIdArray = []


// event listeners
goalsList.addEventListener('click', e => {
  let target = e.target

  if(target.closest('.item-goal')) {
    let targetGoal = target.closest('.item-goal')

    if(targetGoal.matches('.selected')) {
      unselectGoal(targetGoal)
    } else {
      selectGoal(targetGoal)
    }
  }
})

modalBackground.addEventListener('click', e => {
  let target = e.target

  if(target === modalBackground || target.matches('.modal-close')) {
    switchModal()
  } else if(target.matches('#selected-goals-confirm')) {
    createUserGoals()
  }
})

selectedGoals.addEventListener('click', e => {
  let target = e.target

  if(target.matches('#selected-goals-done')) {
    if(selectedIdArray.length > 0) {
      showModalSelected()
      switchModal(true)
    }
  } else if(target.matches('#selected-goals-reset')) {
    // clear the selectedIdArray, remove styling in goals list, update selectedList

    selectedIdArray = [];

    let goals = goalsList.querySelectorAll('.item-goal')

    for(const goal of goals) {
      goal.classList.remove('selected')
    }

    updateSelectedList()
  }
})


// functions

// switching element visibility
function switchModal(modal = false) {
  modalBackground.classList.add('is-hidden')

  if(modal) {
    modalBackground.classList.remove('is-hidden')
  }
}

// showing modal data
function showModalSelected() {
  let selectedItems = selectedList.querySelectorAll('.item-selected')
  let ModalSelectedItems = modalSelectedList.querySelectorAll('.item-selected')
  
  for(const goal of ModalSelectedItems) {
    goal.remove()
  }
  
  for(const goal of selectedItems) {
    modalSelectedList.appendChild(goal.cloneNode(true))
  }
  
}

// selected goals list
function selectGoal(goal) {
  selectedIdArray.push(+goal.dataset.goalId)

  goal.classList.add('selected')

  updateSelectedList()
}

function unselectGoal(goal) {
  let goalIndex = selectedIdArray.indexOf(+goal.dataset.goalId)

  selectedIdArray.splice(goalIndex, 1)

  goal.classList.remove('selected')

  updateSelectedList()
}

function updateSelectedList() {
  let goals = goalsList.querySelectorAll('.item-goal')
  let selectedItems = selectedList.querySelectorAll('.item-selected')
  
  for(const goal of selectedItems) {
    goal.remove()
  }
  
  if(selectedIdArray.length > 0) {
    // create elements
    for(const goalId of selectedIdArray) {
      let item = document.createElement('li')
      let itemTitle = document.createElement('p')
      let itemSkill = document.createElement('span')
    
      item.classList.add('item-selected' , 'py-2' , 'px-4' , 'rounded-md' , 'bg-neutral-200')
      itemSkill.classList.add('text-xs', 'whitespace-nowrap')
    
      // find goal in DOM with corresponding id, grab goal & skill title
      for(const goal of goals) {
        if(+goal.dataset.goalId === goalId) {
          itemTitle.innerText = `${goal.firstElementChild.innerText} `
          itemSkill.innerText = `(${goal.closest('.item-skill').querySelector('.skill-title').innerText})`
          break
        }
      }
      itemTitle.appendChild(itemSkill)
    
      item.appendChild(itemTitle)
    
      selectedList.appendChild(item)
    }
    
    noItemsMessage.classList.add('is-hidden')
  } else {
    noItemsMessage.classList.remove('is-hidden')
  }

  // set selected goals count
  selectedGoalsCount.innerText = `(${selectedIdArray.length})`
}

// CRUD

function createUserGoals() {
  if(selectedIdArray.length <= 0) return

  // send axios request
  axios.post('/user/leerdoelen_kiezen/create', {goals: selectedIdArray})
  .then(response => {
    // redirect to leerdoelen view
    window.location.href = '/user/leerdoelen';
  })
  .catch(error => {
    console.log(error)
  })
}


// onload
updateSelectedList()

axios.get('/user/leerdoelen_kiezen/get_previous')
.then(response => {
  selectPreviousGoals(response.data)
})
.catch(error => {
  console.log(error)
})

function selectPreviousGoals(idArray) {
  let goals = goalsList.querySelectorAll('.item-goal')

  // add 'selected' class to goals
  for(const goalId of idArray) {
    for(const goal of goals) {
      if(+goal.dataset.goalId === goalId) {
        goal.classList.add('selected')
        break
      }
    }
  }
  selectedIdArray = idArray
  updateSelectedList()
}