<x-layouts.admin>

  <x-slot name="assets">
    <script type="module" src="{{ asset('js/views/goalsManageView.js') }}"></script>
  </x-slot>

  <x-slot name="pageTop">
    <h1 class="text-4xl font-medium">Leerdoelen Beheren</h1>
  </x-slot>

  <x-navigationTabs/>

  <section id="goals-list" class="inline-flex flex-col gap-8">

    @foreach ($skills as $skill)

      <div class="item-skill" data-skill-id="{{ $skill->id }}">

        <h2 class="skill-title font-medium text-2xl my-6">{{ $skill->title }}</h2>

        <ul class="flex flex-col gap-4">

          @foreach ($skill->topics as $topic)
        
            <li class="item-topic" data-topic-id="{{ $topic->id }}">
              <div class="inline-block">
                <h3 class="topic-title text-xl font-medium">{{ $topic->title }}</h3>
              </div>

              <!-- list of goals -->
              <ul class="ml-2 my-2 flex flex-col gap-2 font-normal">
                @foreach ($topic->goals as $goal)
                  <li class="item-goal py-2 px-4 flex items-center justify-between gap-20 rounded-md bg-neutral-200 cursor-pointer" data-goal-id="{{ $goal->id }}">

                    <p class="goal-title">{{ $goal->title }}</p>
                    
                    <div class="flex gap-4 items-center">
                      <p class="whitespace-nowrap">{{ $goal->users->where('goal_achieved', 1)->count() }} / {{ $goal->users->count() }}</p>
                      <button class="button-edit-goal py-1 px-4 rounded-md text-sm font-semibold bg-sky-500 hover:bg-sky-400 text-white">Wijzig</button>
                    </div>

                  </li>
                @endforeach
              </ul>

              <button class="button-add-goal w-8 h-8 ml-2 rounded-md font-semibold bg-sky-500 hover:bg-sky-400 text-white">+</button>


            </li>

          @endforeach

        </ul>

      </div>

    @endforeach

  </section>

  <!-- modals -->

  <div id="modal-background" class="is-hidden fixed h-screen w-screen top-0 left-0 flex justify-center items-center bg-neutral-600 bg-opacity-80">

    <div id="modal-view-goal" class="modal is-hidden relative max-w-2xl py-10 px-14 rounded-lg flex flex-col gap-4 bg-white">
      <button class="modal-close absolute top-4 right-4 h-6 w-6 font-semibold">x</button>
      <h2 class="text-2xl font-semibold">Leerdoel Info</h2>
      <p id="view-goal-title"></p>
      <div class="h-0.5 w-full my-4 bg-neutral-200"></div>
      <h3 class="font-medium">Leerdoel van:</h3>
      <ul id="view-goal-users" class="flex flex-wrap gap-2">
        <li class="item-user py-1 px-4 rounded-full bg-neutral-200 text-sm">User</li>
        <li class="item-user py-1 px-4 rounded-full bg-neutral-200 text-sm">User</li>
        <li class="item-user py-1 px-4 rounded-full bg-neutral-200 text-sm">User</li>
        <li class="item-user py-1 px-4 rounded-full bg-neutral-200 text-sm">User</li>
      </ul>
    </div>

    <div id="modal-edit-goal" class="modal is-hidden relative max-w-2xl py-10 px-14 rounded-lg flex flex-col gap-4 bg-white">
      <button class="modal-close absolute top-4 right-4 h-6 w-6 font-semibold">x</button>
      <h2 class="text-2xl font-semibold">Leerdoel Wijzigen</h2>
      <form class="flex flex-col gap-4">
        <div class="flex justify-between">
          <label for="edit-goal-topic">Onderwerp</label>
          <select id="edit-goal-topic" class="w-3/5 p-2 bg-inherit border-2 rounded-md" name="topic">

            @foreach ($skills as $skill)
              <optgroup label="{{ $skill->title }}">
                
                @foreach ($skill->topics as $topic)
                  <option value="{{ $topic->title }}" data-topic-id="{{ $topic->id }}">{{ $topic->title}}</option>
                @endforeach

              </optgroup>
            @endforeach
            
          </select>
        </div>
        <div class="flex justify-between">
          <label for="edit-goal-title">Titel leerdoel</label>
          <textarea id="edit-goal-title" class="w-3/5 p-2 border-2 rounded-md" name="title" cols="30" rows="5"></textarea>
        </div>
      </form>
      <div class="mt-4 flex justify-between">
        <button id="edit-goal-delete" class="py-2 px-4 rounded-md bg-red-500 text-white font-semibold">Verwijder</button>
        <button id="edit-goal-update" class="py-2 px-4 rounded-md bg-green-500 text-white font-semibold">Opslaan</button>
      </div>
    </div>

    <div id="modal-add-goal" class="modal is-hidden relative max-w-2xl py-10 px-14 rounded-lg flex flex-col bg-white">
      <button class="modal-close absolute top-4 right-4 h-6 w-6 font-semibold">x</button>
      <h2 class="text-2xl font-semibold">Leerdoel Toevoegen</h2>
      <p id="add-goal-topic" class="mb-4 text-sm"></p>
      <form class="flex justify-between">
        <label for="add-goal-title">Titel leerdoel</label>
        <textarea id="add-goal-title" class="w-3/5 p-2 border-2 rounded-md" name="title" cols="30" rows="5"></textarea>
      </form>
      <div class="mt-4 flex justify-end">
        <button id="add-goal-create" class="py-2 px-4 rounded-md bg-green-500 text-white font-semibold">Toevoegen</button>
      </div>
    </div>

  </div>
  
  <p id="server-is-busy-message" class="is-hidden">Server is busy...</p>

</x-layouts.admin>