<?php

namespace App\Http\Controllers;

use App\Models\Goal;
use App\Models\User;
use App\Models\Skill;
use App\Models\Topic;
use App\Models\UserGoal;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class UserGoalController extends Controller
{
  // Views

  public function goalsCurrentView()
  {
    // needed info (besides id's):
    // userGoal: goal_achieved, goal_approved, goal_achieved_date
    // goal: title
    // skill: title
    $user = Auth::user();

    // get currently active goals
    $userGoals = UserGoal::getActiveWithSkillTitle($user->id);

    $goalsAreApproved = $userGoals->where('goal_approved', 0)->count() == 0;
  
    return view('user.goals-current', ['goals' => $userGoals,'goals_are_approved' => $goalsAreApproved, 'can_select_goals' => $user->can_select_goals]);
  }

  public function goalsAchievedView()
  {
    // needed info (besides id's):
    // userGoal: goal_achieved, goal_achieved_date
    // goal: title
    // skill: title
    $user = Auth::user();

    $skillsWithGoals = Skill::getWithAchievedGoals($user->id);
  
    return view('user.goals-achieved', ['skills' => $skillsWithGoals, 'can_select_goals' => $user->can_select_goals]);
  }

  public function goalsSelectView()
  {
    // needed info (besides id's):
    // userGoal: goal_achieved
    // goal: title
    // topic: title
    // skill: title
    $user = Auth::user();

    // redirect back if user shouldn't select new goals
    if(!$user->can_select_goals) {
      return back();
    }

    // get all goals that the user has not achieved yet.
    $skills = Skill::getForSelectView($user->id);

    return view('user.goals-select', ['skills' => $skills]);
  }

  // CRUD

  public function getPreviousGoals()
  {
    $user = Auth::user();

    // get unreached goals from previous selection
    $previousUserGoals = UserGoal::where('user_id', $user->id)->where('goal_achieved', 0)->select('goal_id')->get();

    $previousGoalsIdArray = [];

    foreach($previousUserGoals as $userGoal) {
      array_push($previousGoalsIdArray, $userGoal->goal_id);
    }

    return $previousGoalsIdArray;
  }

  public function setNewGoals(Request $request)
  {
    $user = Auth::user();

    $userGoals = UserGoal::select('id', 'user_id', 'goal_id', 'goal_active', 'goal_achieved')->where('user_id', $user->id)->get();

    // set user can_select_goals to 0, when i try to call on auth::user it doesn't work.
    $userTemp = User::find($user->id);

    $userTemp->canNotSelectGoals();

    // delete userGoals that are unachieved && are not reselected
    foreach($userGoals as $userGoal) {
      // dd($userGoals->first()->goal_selected_date);
      if($userGoal->goal_achieved === 0) {
        $isReselected = false;

        foreach($request->goals as $goalId) {
          if($userGoal->goal_id === $goalId) {
            $isReselected = true;
          }
        }

        if(!$isReselected) {
          $userGoal->delete();
        }
      } else if($userGoal->goal_active === 1) {
        $userGoal->deactivate();
      }
    }

    // if userGoal already exists, call goal->reselect(), if new, create new userGoal
    foreach($request->goals as $goalId) {
      $targetGoal = $userGoals->where('goal_id', $goalId)->first();
      
      if($targetGoal) {
        if($targetGoal->goal_achieved === 1) continue;

        $targetGoal->reselectGoal();
        
      } else {

        UserGoal::create([
          'user_id' => $user->id,
          'goal_id' => $goalId,
          'goal_active' => 1,
          'goal_approved' => 0,
          'goal_achieved' => 0,
          'goal_selected_date' => now(),
        ]);

      }
    }
  }
}
