<?php

namespace App\Models;

use App\Models\Topic;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class Skill extends Model
{
  public $timestamps = false;

  protected $fillable = [
    'title',
  ];


  // relations

  public function topics()
  {
    return $this->hasMany(Topic::class)->select('id', 'skill_id', 'title');
  }

  // query methods
  
  public static function getForManageView()
  {
    // query skills table
    $skills = Skill::select('id', 'title')->with(['topics' => function($query) {
      // query topics table
      $query->select('id', 'skill_id', 'title')->with(['goals' => function($query) {
        // query goals table
        $query->select('id', 'topic_id', 'title')->with(['users' => function($query) {
          // query userGoals table
          $query->select('goal_id', 'goal_achieved');
        }]);
      }]);
    }])->get();

    return $skills;
  }

  public static function getForSelectView($userId)
  {
    // this query needs to:
    // see if UserGoal's goal_achieved = 1,
    // if so, send something back to Goals query
    // have a where() in Goals to not include goals the user has already achieved

    // query skills table
    $skills = Skill::select('id', 'title')->with(['topics' => function($query) use($userId){
      // query topics table
      $query->select('id', 'skill_id', 'title')->with(['goals' => function($query) use($userId) {
        // query goals table
        $query->select('id', 'topic_id', 'title')
        
        ->where(function($query) use($userId) {
          // dont select goals where UserGoal goal_achieved = 1
          $query->select('goal_achieved')->from('user_goals')->where('user_goals.user_id', $userId)->whereColumn('user_goals.goal_id', 'goals.id');
        }, [0])
        ->orWhereNotExists(function($query) use($userId) {
          $query->select('goal_achieved')->from('user_goals')->where('user_goals.user_id', $userId)->whereColumn('user_goals.goal_id', 'goals.id');
        })

        ->with(['users' => function($query) {
          // query userGoals table
          $query->select('goal_id', 'user_id', 'goal_achieved');
        }]);
      }]);
    }])->get();

    return $skills;
  }

  public static function getWithAchievedGoals($userId)
  {
    // query skills table
    $skills = Skill::select('id', 'title')->with(['topics' => function($query) use($userId){
      // query topics table
      $query->select('id', 'skill_id')->with(['goals' => function($query) use($userId) {
        // query goals table
        $query->select('id', 'topic_id', 'title')

        ->where(function($query) use($userId) {
          // only select goals with userGoal achieved 1
          $query->select('goal_achieved')->from('user_goals')->where('user_goals.user_id', $userId)->whereColumn('user_goals.goal_id', 'goals.id');
        }, [1])

        ->with(['users' => function($query) use($userId) {
          // query userGoals table
          $query->select('goal_id', 'user_id', 'goal_achieved', 'goal_achieved_date')->where('user_id', $userId);
        }]);
      }]);
    }])->get();

    return $skills;
  }


  // methods

}
