<?php

namespace App\Models;

use App\Models\UserGoal;
use Laravel\Sanctum\HasApiTokens;
use Illuminate\Notifications\Notifiable;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;

class User extends Authenticatable
{
  use HasApiTokens, HasFactory, Notifiable;

  /**
   * The attributes that are mass assignable.
   *
   * @var array<int, string>
   */
  protected $fillable = [
    'name',
    'username',
    'password',
    'can_select_goals',
    'is_admin',
  ];

  /**
   * The attributes that should be hidden for serialization.
   *
   * @var array<int, string>
   */
  protected $hidden = [
    'password',
    'remember_token',
  ];


  // relations

  public function goals()
  {
    return $this->hasMany(UserGoal::class);
  }

  // query methods

  public static function getWithActiveGoals()
  {
    $users = User::select('id', 'name')->where('is_admin', 0)->with(['goals' => function($query) {
      $query->where('goal_active', 1)->select('user_id', 'goal_achieved');
    }])->get();

    return $users;
  }

  public static function getWithActiveGoalsAndGoalAndSkillTitles() {
    $users = User::select('id', 'name', 'can_select_goals')->where('is_admin', 0)

    ->with(['goals' => function($query) {
      $query->where('goal_active', 1)->select('id', 'user_id', 'goal_id', 'goal_achieved', 'goal_approved')

      ->with(['goal' => function($query) {
        $query->select('id', 'topic_id', 'title')

        ->with(['topic' => function($query) {
          $query->select('id', 'skill_id')
          
          ->with(['skill' => function($query) {
            $query->select('id', 'title');
          }]);
        }]);
      }]);
    }])
    ->get();

    return $users;
  }

  // methods

  public function toggleCanSelectGoals()
  {
    // toggle if user can select goals
    $this->can_select_goals = !$this->can_select_goals;

    $this->save();
  }

  public function canSelectGoals()
  {
    $this->can_select_goals = 1;

    $this->save();
  }

  public function canNotSelectGoals()
  {
    $this->can_select_goals = 0;

    $this->save();
  }

}
