<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class UserGoal extends Model
{
  use HasFactory;

  public $timestamps = false;

  protected $fillable = [
    'user_id',
    'goal_id',
    'goal_active',
    'goal_approved',
    'goal_achieved',
    'goal_achieved_date',
    'goal_selected_date',
  ];


  // relations

  public function user()
  {
    return $this->belongsTo(User::class);
  }

  public function goal()
  {
    return $this->belongsTo(Goal::class);
  }

  // query methods

  public static function getActiveWithSkillTitle($userId)
  {
    $userGoals = UserGoal::select('id', 'goal_id', 'goal_approved', 'goal_achieved', 'goal_achieved_date')->where('user_id', $userId)->where('goal_active', 1)->with(['goal' => function($query) {
      $query->select('id', 'topic_id', 'title')->with(['topic' => function($query) {
        $query->select('id', 'skill_id')->with(['skill' => function($query) {
          $query->select('id', 'title');
        }]);
      }]);
    }])->get();

    return $userGoals;
  }
  

  // methods

  public function reselectGoal()
  {
    $this->goal_approved = 0;
    $this->goal_selected_date = now();

    $this->save();
  }

  public function approveGoal()
  {
    $this->goal_approved = 1;

    $this->save();
  }

  public function achieveGoal()
  {
    if($this->goal_achieved == 0) {
      $this->goal_achieved = 1;
      $this->goal_achieved_date = now();
  
      $this->save();
    }
  }

  public function unachieveGoal()
  {
    if($this->goal_achieved == 1) {
      $this->goal_achieved = 0;
      $this->goal_achieved_date = null;

      $this->save();
    }
  }

  public function deactivate()
  {
    $this->goal_active = 0;

    $this->save();
  }

}
