"use strict"

import "../components/tabNavigation.js"
import { serverIsBusy, toggleServerIsBusy } from "../components/requestLimiter.js"

// Declarations
const goalsList = document.querySelector('#goals-list')

const modalBackground = document.querySelector('#modal-background')

const modalView = document.querySelector('#modal-view-goal')
const modalViewTitle = document.querySelector('#view-goal-title')
const modalViewUsers = document.querySelector('#view-goal-users')

const modalAdd = document.querySelector('#modal-add-goal')
const modalAddTopic = document.querySelector('#add-goal-topic')
const modalAddTitle = document.querySelector('#add-goal-title')

const modalEdit = document.querySelector('#modal-edit-goal')
const modalEditTopic = document.querySelector('#edit-goal-topic')
const modalEditTitle = document.querySelector('#edit-goal-title')

let goalsData = []

let currentTopicId = null
let currentGoalId = null


// Event listeners
modalBackground.addEventListener('click', e => {
  let target = e.target

  if(target === modalBackground || target.matches('.modal-close')) {
    resetCurrentData()
    switchModal()
  }

  // modal CRUD buttons
  if(target.matches('#add-goal-create')) {
    // call create method
    createGoal(currentTopicId, modalAddTitle.value)

  } else if(target.matches('#edit-goal-update')) {
    // call update method
    let topicOptions = modalEditTopic.querySelectorAll('option')
    let topicId = null

    for(const option of topicOptions) {
      if(option.selected) {
        topicId = +option.dataset.topicId
        break
      }
    }

    updateGoal(currentGoalId, topicId, modalEditTitle.value)
    
  } else if(target.matches('#edit-goal-delete')) {
    // call delete method
    deleteGoal(currentGoalId)
  }
})

goalsList.addEventListener('click', e => {
  let target = e.target

  if(target.matches('.button-edit-goal')) {
    currentGoalId = +target.closest('.item-goal').dataset.goalId
    showModalEdit(currentGoalId)
    switchModal('edit')

  } else if(target.closest('.item-goal')) {
    currentGoalId = +target.closest('.item-goal').dataset.goalId
    showModalView(currentGoalId)
    switchModal('view')

  } else if(target.matches('.button-add-goal')) {
    currentTopicId = +target.parentElement.dataset.topicId
    showModalAdd(target.parentElement)
    switchModal('add')
  }
})


// Functions

// Switching element visibility
function switchModal(modal) {
  modalBackground.classList.add('is-hidden')
  modalView.classList.add('is-hidden')
  modalEdit.classList.add('is-hidden')
  modalAdd.classList.add('is-hidden')

  if(modal != undefined) {
    modalBackground.classList.remove('is-hidden')
    if(modal === 'view') {
      modalView.classList.remove('is-hidden')
    } else if(modal === 'edit') {
      modalEdit.classList.remove('is-hidden')
    } else if(modal === 'add') {
      modalAdd.classList.remove('is-hidden')
    }
  }
}

// Showing modal data
function showModalView(goalId) {
  let goal = goalsData.find(goal => goal.id === goalId)

  modalViewTitle.innerText = goal.title
  
  // clear users list
  let usersList = modalViewUsers.querySelectorAll('li')

  for(const user of usersList) {
    user.remove()
  }

  // add users to list
  for(const user of goal.users) {
    let liElement = document.createElement('li')

    liElement.classList.add('item-user', 'py-1', 'px-4', 'rounded-full', 'bg-neutral-200', 'text-sm')

    if(user.goal_achieved) liElement.classList.add('achieved')

    liElement.innerText = user.name

    modalViewUsers.appendChild(liElement)
  }
}

function showModalEdit(goalId) {
  let goal = goalsData.find(goal => goal.id === goalId)
  let topicOptions = modalEditTopic.querySelectorAll('option')

  // select current option of topic select
  for(const option of topicOptions) {
    if(option.value === goal.topic) option.selected = true
  }

  modalEditTitle.value = goal.title
}

function showModalAdd(topicElement) {
  modalAddTopic.innerText = `${
    topicElement.closest('.item-skill').querySelector('.skill-title').innerText
  } - ${
    topicElement.querySelector('.topic-title').innerText
  }`

  modalAddTitle.value = ''
}

// Updating changes in front-end
function updateChangesCreate(data) {
  // create elements
  let goal = document.createElement('li')
  let goalTitle = document.createElement('p')
  let goalDiv = document.createElement('div')
  let goalUserCount = document.createElement('p')
  let goalEditButton = document.createElement('button')

  // add data
  goal.dataset.goalId = data.id

  // add classes
  goal.classList.add('item-goal', 'py-2', 'px-4', 'flex', 'items-center', 'justify-between', 'gap-20', 'rounded-md', 'bg-neutral-200')
  goalTitle.classList.add('goal-title')
  goalDiv.classList.add('flex', 'gap-4', 'items-center')
  goalEditButton.classList.add('button-edit-goal', 'py-1', 'px-4', 'rounded-md', 'text-sm', 'font-semibold', 'bg-sky-500', 'hover:bg-sky-400', 'text-white')

  // fill in content
  goalTitle.innerText = data.title
  goalUserCount.innerText = '0 / 0'
  goalEditButton.innerText = 'Wijzig'
  
  // append
  goalDiv.appendChild(goalUserCount)
  goalDiv.appendChild(goalEditButton)
  goal.appendChild(goalTitle)
  goal.appendChild(goalDiv)

  const topicElement = goalsList.querySelector(`[data-topic-id = '${data.topic_id}']`)
  const targetGoalList = topicElement.querySelector('ul')
  targetGoalList.appendChild(goal)
  
  // update data
  let topicTitle = topicElement.querySelector('.topic-title').innerText
  let newGoal = {id: data.id, title: data.title, topic: topicTitle, users: []}
  goalsData.push(newGoal)

}

function updateChangesUpdate(data) {
  let goal = goalsData.find(goal => goal.id === data.id)
  const goalElement = goalsList.querySelector(`[data-goal-id = '${data.id}']`)

  // change title
  if(goal.title != data.title) {
    goalElement.querySelector('.goal-title').innerText = data.title
  }
  
  // change parent
  const topicElement = goalsList.querySelector(`[data-topic-id = '${data.topic_id}']`)
  let topicTitle = topicElement.querySelector('.topic-title').innerText
  if(goal.topic != topicTitle) {
    const targetGoalList = topicElement.querySelector('ul')
    targetGoalList.appendChild(goalElement)
  }
  
  // update data
  goal.title = data.title
  goal.topic = topicTitle

}

function updateChangesDelete(goalId) {
  const goalElement = goalsList.querySelector(`[data-goal-id = '${goalId}']`)
  let goalIndex = goalsData.findIndex(goal => goal.id === goalId)

  goalElement.remove()
  goalsData.splice(goalIndex, 1)
}

// Utility
function resetCurrentData() {
  currentTopicId = null
  currentGoalId = null
}

// CRUD
async function createGoal(topicId, title) {
  if(serverIsBusy === true || !topicId || title === '') return
  resetCurrentData()
  switchModal()
  
  // send axios request
  toggleServerIsBusy(true)
  await axios.post('/admin/leerdoelen_beheer/create', {topicId: topicId, title: title})
  .then(response => {
    updateChangesCreate(response.data)
    toggleServerIsBusy(false)
  })
  .catch(error => {
    console.log(error)
  })
}

async function updateGoal(goalId, topicId, title) {
  if(serverIsBusy === true || !goalId || !topicId || title === '') return
  resetCurrentData()
  switchModal()
  
  // send axios request
  toggleServerIsBusy(true)
  await axios.put('/admin/leerdoelen_beheer/update', {goalId: goalId, topicId: topicId, title: title})
  .then(response => {
    updateChangesUpdate(response.data)
    toggleServerIsBusy(false)
  })
  .catch(error => {
    console.log(error)
  })
}

async function deleteGoal(goalId) {
  if(serverIsBusy === true || !goalId) return
  resetCurrentData()
  switchModal()
  
  // send axios request
  toggleServerIsBusy(true)
  await axios.delete('/admin/leerdoelen_beheer/delete', {data: {goalId: goalId}})
  .then(response => {
    updateChangesDelete(response.data)
    toggleServerIsBusy(false)
  })
  .catch(error => {
    console.log(error)
  })
}


// Onload
toggleServerIsBusy(true)
axios.get('/admin/leerdoelen_beheer/read')
.then(response => {
  goalsData = changeDataFormat(response.data)
  console.log(goalsData)
  toggleServerIsBusy(false)
})
.catch(error => {
  console.log(error)
})

function changeDataFormat(data) {
  // make a few changes in the loaded data, because the query changed.

  for(const goal of data) {
    goal.topic = goal.topic.title
    
    for(const user of goal.users) {
      user.name = user.user.name
      user.goal_achieved = +user.goal_achieved === 1
    }
  }

  return data;
}