"use strict"

import { serverIsBusy, toggleServerIsBusy } from "../components/requestLimiter.js"

// Declarations
const usersGrid = document.querySelector('#users-grid')

const modalBackground = document.querySelector('#modal-background')

const modalUserName = document.querySelector('#user-name')
const modalUserCanSelectGoals = document.querySelector('#user-can-select-goals')
const modalUserApproveGoals = document.querySelector('#user-approve-goals')
const modalUserGoals = document.querySelector('#user-goals')
const noItemsMessage = document.querySelector('#user-no-items-message')

let currentUser
let currentUserChanges = {canSelectGoals: null, goalsApproved: null, goals: []}

// data to get: 
//  user[id, name, can_select_goals]
//    userGoal[id, goal_achieved, goal_approved]
//      goal[title, skill->title]
let userData = []

// Event listeners
usersGrid.addEventListener('click', e => {
  let target = e.target

  if(target.matches('.message') || target.matches('.message-box')) return

  if(target.closest('.item-user')) {
    setCurrentUser(target.closest('.item-user').dataset.userId)
    showModalUser(currentUser)
    switchModal(true);
  }

})

modalBackground.addEventListener('click', e => {
  let target = e.target

  if(target === modalBackground || target.matches('.modal-close')) {
    resetCurrentChanges()
    switchModal()

  } else if(target === modalUserCanSelectGoals) {
    changeSelectGoals(currentUser)

  } else if(target === modalUserApproveGoals) {
    changeApproveGoals(currentUser)

  } else if(target.matches('.user-achieve-goal')) {
    let goal = currentUser.goals.find(goal => goal.id === +target.closest('.item-goal').dataset.goalId)
    changeAchieveGoal(goal, target)

  } else if(target.matches('#user-update')) {
    updateChanges(currentUser, currentUserChanges)
    
  } else if(target.matches('.user-delete-goal')) {
    let goalItem = target.closest('.item-goal')
    let goal = currentUser.goals.find(goal => goal.id === +goalItem.dataset.goalId)
    changeDeleteGoal(goal, goalItem)
  }
})


// Functions

// switching element visibility
function switchModal(modal = false) {
  modalBackground.classList.add('is-hidden')

  if(modal) {
    modalBackground.classList.remove('is-hidden')
  }
}


// showing modal data
function showModalUser(user) {

  // clear goals list
  let goalsList = modalUserGoals.querySelectorAll('.item-goal')

  for(const goal of goalsList) {
    goal.remove()
  }

  modalUserName.innerText = user.name
  // set button states
  if(user.canSelectGoals) {
    modalUserCanSelectGoals.classList.add('selected')
  } else {
    modalUserCanSelectGoals.classList.remove('selected')
  }
  if(user.goalsApproved) {
    modalUserApproveGoals.classList.add('is-hidden')
  } else {
    modalUserApproveGoals.classList.remove('is-hidden', 'selected')
  }

  // make list of user's goals
  for(const goal of user.goals) {
    // create elements
    let goalItem = document.createElement('li')
    let goalTitle = document.createElement('p')
    let goalSkillTitle = document.createElement('span')
    let goalButtonAchieve = document.createElement('button')
    let goalButtonDelete = document.createElement('button')

    // add data
    goalItem.dataset.goalId = goal.id

    // add classes
    goalItem.classList.add('item-goal' , 'py-2' , 'px-4' , 'flex', 'items-center', 'gap-4', 'rounded-md' , 'bg-neutral-200')
    goalTitle.classList.add('flex-1')
    goalSkillTitle.classList.add('text-xs')
    goalButtonAchieve.classList.add('user-achieve-goal', 'py-1', 'px-4', 'text-sm', 'font-semibold', 'text-neutral-700', 'rounded-md', 'border-2', 'border-neutral-400')
    if(!user.goalsApproved) {
      goalButtonAchieve.classList.add('is-hidden')
    }
    if(goal.goalAchieved) {
      goalButtonAchieve.classList.add('selected')
    }
    goalButtonDelete.classList.add('user-delete-goal', 'h-6', 'w-6', 'font-semibold', 'text-neutral-700')

    if(goal.title === "Leerdoel bestaat niet meer") {
      goalSkillTitle.classList.add('warning')
    }

    // fill in content
    goalTitle.innerText = `${goal.title} `
    goalSkillTitle.innerText = `(${goal.skill})`
    goalButtonAchieve.innerText = 'Behaald'
    goalButtonDelete.innerText = 'x'

    // append
    goalTitle.appendChild(goalSkillTitle)

    goalItem.appendChild(goalTitle)
    goalItem.appendChild(goalButtonAchieve)
    goalItem.appendChild(goalButtonDelete)

    modalUserGoals.appendChild(goalItem)
  }

  // display message in case there are no goals
  if(user.goals.length > 0) {
    noItemsMessage.classList.add('is-hidden')
  } else {
    noItemsMessage.classList.remove('is-hidden')
  }
}


// changing user data
function changeSelectGoals(user) {
  if(currentUserChanges.canSelectGoals === user.canSelectGoals) {
    currentUserChanges.canSelectGoals = !user.canSelectGoals
  } else {
    currentUserChanges.canSelectGoals = user.canSelectGoals
  }

  if(currentUserChanges.canSelectGoals) {
    modalUserCanSelectGoals.classList.add('selected')
  } else {
    modalUserCanSelectGoals.classList.remove('selected')
  }
}

function changeApproveGoals(user) {
  if(currentUserChanges.goalsApproved === user.goalsApproved) {
    currentUserChanges.goalsApproved = !user.goalsApproved
  } else {
    currentUserChanges.goalsApproved = user.goalsApproved
  }

  if(currentUserChanges.goalsApproved) {
    modalUserApproveGoals.classList.add('selected')
  } else {
    modalUserApproveGoals.classList.remove('selected')
  }
}

function changeAchieveGoal(goal, targetObject) {
  let changesGoal = currentUserChanges.goals.find(obj => obj.id === goal.id)

  if(changesGoal.goalAchieved === goal.goalAchieved) {
    changesGoal.goalAchieved = !goal.goalAchieved
  } else {
    changesGoal.goalAchieved = goal.goalAchieved
  }

  if(changesGoal.goalAchieved) {
    targetObject.classList.add('selected')
  } else {
    targetObject.classList.remove('selected')
  }
}

function changeDeleteGoal(goal, goalItem) {
  let changesGoal = currentUserChanges.goals.find(obj => obj.id === goal.id)

  if(changesGoal.isDeleted === false) {
    changesGoal.isDeleted = true
    goalItem.classList.add('is-deleted')
  } else {
    changesGoal.isDeleted = false
    goalItem.classList.remove('is-deleted')
  }
}

async function updateChanges(user, inputChanges) {
  // if server is busy. return IMMEDIATELY! GET OUT!
  if(serverIsBusy === true) return

  // close modal
  switchModal()

  // clone the changes, so when you close the modal it doesn't mess up
  let changes = JSON.parse(JSON.stringify(inputChanges));

  // reset the currentChanges object
  resetCurrentChanges()

  // call axios function, if succeeded make changes permanent in js userData
  if(await axiosUpdateChanges(user.id, changes)) {
    user.canSelectGoals = changes.canSelectGoals
    user.goalsApproved = changes.goalsApproved
    // update goals data
    for(let i = 0; i < user.goals.length; i++) {
      let goal = user.goals[i]
      // if goal got deleted in changes, delete goal from userData
      const changedGoal = changes.goals.find(obj => obj.id === goal.id)

      if(changedGoal.isDeleted === true) {
        user.goals.splice(i, 1)
        i--
        continue
      }

      goal.goalAchieved = changedGoal.goalAchieved
    }

    // update goals/achieved counts in users list
    let itemUser = usersGrid.querySelector(`[data-user-id = '${user.id}']`)
    let activeGoalsCount = itemUser.querySelector('.active-goals-count')
    let achievedGoalsCount = itemUser.querySelector('.achieved-goals-count')

    activeGoalsCount.innerText = user.goals.length

    let amountOfAchievedGoals = 0
    for(const goal of user.goals) {
      if(goal.goalAchieved) {
        amountOfAchievedGoals++
      }
    }
    achievedGoalsCount.innerText = amountOfAchievedGoals

    // update what message is displayed
    updateMessage(user)
  }
}


// Setting and updating messages
function updateMessage(user) {
  let itemUser = usersGrid.querySelector(`[data-user-id = '${user.id}']`)
  let messageElement = itemUser.querySelector('.message')

  let message = null

  if(user.goalsApproved === false && user.goals.length > 0) {
    message = createMessageContent('goalsUnapproved')

  } else if(user.canSelectGoals === true) {
    message = createMessageContent('canSelectGoals')

  } else if(user.goals.length === 0) {
    message = createMessageContent('noGoals')
  }

  // if there is a deleted goal in the list, override message for this.
  messageElement.classList.remove('warning')
  for(const goal of user.goals) {
    if(goal.title === "Leerdoel bestaat niet meer") {
      message = createMessageContent('deletedGoal')
      messageElement.classList.add('warning')
    }
  }

  if(message === null) {
    messageElement.classList.add('is-hidden')
  } else {
    messageElement.innerText = message.text
    messageElement.classList.remove('is-hidden')
  }
}

function updateAllMessages() {
  let userItems = usersGrid.querySelectorAll('.item-user')
  
  for(const user of userData) {
    let messageElement = null

    for(const userItem of userItems) {
      if(+userItem.dataset.userId === user.id) {
        messageElement = userItem.querySelector('.message')
        break
      }
    }

    let message = null

    if(user.goalsApproved === false && user.goals.length > 0) {
      message = createMessageContent('goalsUnapproved')

    } else if(user.canSelectGoals === true) {
      message = createMessageContent('canSelectGoals')

    } else if(user.goals.length === 0) {
      message = createMessageContent('noGoals')
    }

    // if there is a deleted goal in the list, override message for this.
    for(const goal of user.goals) {
      if(goal.title === "Leerdoel bestaat niet meer") {
        message = createMessageContent('deletedGoal')
        messageElement.classList.add('warning')
      }
    }

    if(message === null) {
      messageElement.classList.add('is-hidden')
    } else {
      messageElement.innerText = message.text
      messageElement.classList.remove('is-hidden')
    }
  }
}

function createMessageContent(type = null) {
  if(type === null) return null

  let message = {text: ''}

  switch (type) {
    case 'noGoals':
      message.text = 'Nog geen leerdoelen'
      break
    case 'canSelectGoals':
      message.text = 'Kan nieuwe leerdoelen kiezen'
      break
    case 'goalsUnapproved':
      message.text = 'Leerdoelen nog niet goedgekeurd'
      break
    case 'deletedGoal':
      message.text = 'Heeft verwijderde leerdoelen'
      break
  }
  return message
}

// Utility
function setCurrentUser(userId) {
  currentUser = userData.find(user => user.id === +userId)

  // set currentUserChanges in this function too
  currentUserChanges.canSelectGoals = currentUser.canSelectGoals
  currentUserChanges.goalsApproved = currentUser.goalsApproved
  
  for(const goal of currentUser.goals) {
    currentUserChanges.goals.push({id: goal.id, goalAchieved: goal.goalAchieved, isDeleted: false})
  }
}

function resetCurrentChanges() {
  // set all keys in currentUserChanges to value null
  currentUserChanges.canSelectGoals = null
  currentUserChanges.goalsApproved = null
  currentUserChanges.goals = []
}


// CRUD

// send request with all currentUserChanges
async function axiosUpdateChanges(userId, changes) {
  toggleServerIsBusy(true)
  let hasSucceeded = false

  await axios.put('/admin/home/updateUserData', {userId: userId, data: changes})
  .then(response => {
    hasSucceeded = true
  })
  .catch(error => {
    console.log(error)
  })

  toggleServerIsBusy(false)
  return hasSucceeded
}

// Onload
toggleServerIsBusy(true)
axios.get('/admin/home/read')
.then(response => {
  userData = changeDataFormat(response.data)
  console.log(userData)
  updateAllMessages()
  toggleServerIsBusy(false)
})
.catch(error => {
  console.log(error)
})

function changeDataFormat(data) {
  // make a few changes in the loaded data, because the query changed.

  let newData = data.map((inUser) => {
    let user = {};
    
    user.id = inUser.id
    user.name = inUser.name
    user.canSelectGoals = +inUser.can_select_goals === 1

    let goalsAreApproved = true
    let userGoals = []

    for(const inGoal of inUser.goals) {
      let goal = {}

      goal.id = inGoal.id
      if(inGoal.goal != null) {
        goal.title = inGoal.goal.title
        goal.skill = inGoal.goal.topic.skill.title

      } else {
        goal.title = "Leerdoel bestaat niet meer"
        goal.skill = "verwijder dit als leerdoel"
      }
      goal.goalAchieved = +inGoal.goal_achieved === 1

      userGoals.push(goal)

      if(+inGoal.goal_approved === 0) goalsAreApproved = false
    }

    user.goalsApproved = goalsAreApproved
    user.goals = userGoals

    return user;
  })

  return newData;
}